searchData={"items":[{"type":"module","title":"KubernetesHealthCheck","doc":"Health check with Kubernetes semantics.\n\nThis module is called by `KubernetesHealthCheck.Plug` to perform\napplication-specific logic.\n\nTo avoid bringing in excess dependencies, this module is a dummy that always\nreturns `:ok`. You should implement your own functions that perform useful checks.","ref":"KubernetesHealthCheck.html"},{"type":"function","title":"KubernetesHealthCheck.basic/0","doc":"Basic health check.\n\nThis is a sanity check that the app is running and responding to requests.\nIt should always succeed.","ref":"KubernetesHealthCheck.html#basic/0"},{"type":"function","title":"KubernetesHealthCheck.liveness/0","doc":"Check if the app is alive and working properly.\n\nThis returns app status for the Kubernetes `livenessProbe`.\nKubernetes continuously checks if the app is alive and working as expected.\nIf it crashes or becomes unresponsive for a specified period of time,\nKubernetes kills and replaces the container.\n\nThis check should be lightweight, only determining if the server is\nresponding to requests and can connect to the DB.","ref":"KubernetesHealthCheck.html#liveness/0"},{"type":"function","title":"KubernetesHealthCheck.readiness/0","doc":"Check if app should be serving public traffic.\n\nThis returns app status for the Kubernetes `readinessProbe`.\nKubernetes continuously checks if the app should serve traffic. If the\nreadiness probe fails, Kubernetes doesn't kill and restart the container,\ninstead it marks the pod as \"unready\" and stops sending traffic to it, e.g.\nin the ingress.\n\nThis is useful to temporarily stop serving requests. For example, if the app\ngets a timeout connecting to a back end service, it might return an error for\nthe readiness probe. After multiple failed attempts, it would switch to\nreturning false for the `livenessProbe`, triggering a restart.\n\nSimilarly, the app might return an error if it is overloaded, shedding\ntraffic until it has caught up.","ref":"KubernetesHealthCheck.html#readiness/0"},{"type":"function","title":"KubernetesHealthCheck.startup/0","doc":"Check if the app has finished booting up.\n\nThis returns app status for the Kubernetes `startupProbe`.\nKubernetes checks this probe repeatedly until it returns a successful\nresponse. After that, Kubernetes switches to executing the other two probes.\nIf the app fails to successfully start before the `failureThreshold` time is\nreached, Kubernetes kills the container and restarts it.\n\nFor example, this check might return OK when the app has started the\nweb-server, connected to a DB, connected to external services, and performed\ninitial setup tasks such as loading a large cache.","ref":"KubernetesHealthCheck.html#startup/0"},{"type":"type","title":"KubernetesHealthCheck.check_return/0","doc":"","ref":"KubernetesHealthCheck.html#t:check_return/0"},{"type":"module","title":"KubernetesHealthCheck.Plug","doc":"Plug to return health check results.\n\nIt calls the app module which does the actual checking.\n\nFollowing is an example Kubernetes deployment yaml configuration:\n\n```yaml\nstartupProbe:\n  httpGet:\n    path: /healthz/startup\n    port: http\n  periodSeconds: 3\n  failureThreshold: 5\n\nlivenessProbe:\n  httpGet:\n    path: /healthz/liveness\n    port: http\n  periodSeconds: 10\n  failureThreshold: 6\n\nreadinessProbe:\n  httpGet:\n    path: /healthz/readiness\n    port: http\n  periodSeconds: 10\n  failureThreshold: 1\n```","ref":"KubernetesHealthCheck.Plug.html"},{"type":"module","title":"Installation - KubernetesHealthCheck.Plug","doc":"Add the plug to your endpoint or router.\nIt whould normally be placed above the logger to avoid noise in your logs\nfrom health checks.\n\n```\nplug KubernetesHealthCheck.Plug,\n  mod: Example.Health,\n  base_path: \"/healthz\"\n```","ref":"KubernetesHealthCheck.Plug.html#module-installation"},{"type":"module","title":"Init Options - KubernetesHealthCheck.Plug","doc":"- `:mod` - Callback module which implements the health checks for the app, default `KubernetesHealthCheck`\n- `:base_path` - Base request_path for health checks, default `/healthz`\n- `:startup_path` - Path for startup check, default ` /startup`\n- `:liveness_path` - Path for liveness check, default ` /liveness`\n- `:readiness_path` - Path for readiness check, default ` /readiness`","ref":"KubernetesHealthCheck.Plug.html#module-init-options"},{"type":"function","title":"KubernetesHealthCheck.Plug.call/2","doc":"","ref":"KubernetesHealthCheck.Plug.html#call/2"},{"type":"function","title":"KubernetesHealthCheck.Plug.init/1","doc":"","ref":"KubernetesHealthCheck.Plug.html#init/1"},{"type":"extras","title":"kubernetes_health_check","doc":"# kubernetes_health_check\n\nHealth check Plug with Kubernetes semantics.\n\nKubernetes has well defined semantics for how health checks should behave,\ndistinguishing between between startup, liveness, and readiness:\n\n**Liveness** is the core health check. It determines whether the app is alive\nand able to respond to requests. It should be relatively fast, as it is called\nfrequently, but should include checks for dependencies, e.g. whether the app\ncan connect to a database or back end service. If the liveness check fails for\na specified period, Kubernetes kills and replaces the instance.\n\n**Startup** checks whether the app has finished booting up. It is useful when\nthe app may take significant time to start, e.g. because it is loading data\nfrom a cache. Separating this from liveness allows us to use different\ntimeouts, rather than making the liveness timeout long enough to support\nstartup. Once startup has completed successfully, Kubernetes does not call it\nagain, it uses the liveness check.\n\n**Readiness** checks whether the app should receive requests. Kubernetes uses\nit to decide whether to route traffic to the the instance. If the readiness\nprobe fails, Kubernetes doesn't kill and restart the container, instead it\nmarks the pod as \"unready\" and stops sending traffic to it, e.g. in the\ningress. It is useful to temporarily stop serving traffic, e.g. when the\ninstance is overloaded or it has transient problems connecting to a back end\nservice.\n\nSee this blog post for more background:\nhttps://www.cogini.com/blog/kubernetes-health-checks-for-elixir-apps/\n\nLinks:\n\n* https://kubernetes.io/docs/tasks/configure-pod-container/configure-liveness-readiness-startup-probes/\n* https://shyr.io/blog/kubernetes-health-probes-elixir\n\nFollowing is an example Kubernetes deployment yaml configuration:\n\n```yaml\n  startupProbe:\n    httpGet:\n      path: /healthz/startup\n      port: http\n    periodSeconds: 3\n    failureThreshold: 5\n\n  livenessProbe:\n    httpGet:\n      path: /healthz/liveness\n      port: http\n    periodSeconds: 10\n    failureThreshold: 6\n\n  readinessProbe:\n    httpGet:\n      path: /healthz/readiness\n      port: http\n    periodSeconds: 10\n    failureThreshold: 1\n  ```","ref":"readme.html"},{"type":"extras","title":"Installation - kubernetes_health_check","doc":"Add the package to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:kubernetes_health_check, \"~> 0.7.0\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Usage - kubernetes_health_check","doc":"Add `KubernetesHealthCheck.Plug` to your endpoint or router.\nPlace it at the very top to avoid noise in your logs from health checks.\n\n```elixir\nplug KubernetesHealthCheck.Plug,\n  mod: Foo.Health,\n  base_path: \"/healthz\"\n```\n\nOptions:\n\n* `:mod` - Callback module which implements the health checks for the app, default `KubernetesHealthCheck`\n* `:base_path` - Base request_path for health checks, default `/healthz`\n* `:startup_path` - Path for startup check, default ` /startup`\n* `:liveness_path` - Path for liveness check, default ` /liveness`\n* `:readiness_path` - Path for readiness check, default ` /readiness`\n\nAdd a module which provides the app-specific health checks.\nFollowing is an example:\n\n```elixir\ndefmodule Example.Health do\n  @moduledoc \"\"\"\n  Collect app status for Kubernetes health checks.\n  \"\"\"\n  alias Example.Repo\n\n  @app :example\n  @repos Application.compile_env(@app, :ecto_repos) || []\n\n  @type check_return ::\n          :ok\n          | {:error, {status_code :: non_neg_integer(), reason :: binary()}}\n          | {:error, reason :: binary()}\n\n  @doc \"\"\"\n  Check if the app has finished booting up.\n\n  This returns app status for the Kubernetes `startupProbe`.\n  Kubernetes checks this probe repeatedly until it returns a successful\n  response. After that, Kubernetes switches to executing the other two probes.\n  If the app fails to successfully start before the `failureThreshold` time is\n  reached, Kubernetes kills the container and restarts it.\n\n  For example, this check might return OK when the app has started the\n  web-server, connected to a DB, connected to external services, and performed\n  initial setup tasks such as loading a large cache.\n  \"\"\"\n  @spec startup :: check_return()\n  def startup do\n    # Return error if there are available migrations which have not been executed.\n    # This supports deployment to AWS ECS using the following strategy:\n    # https://engineering.instawork.com/elegant-database-migrations-on-ecs-74f3487da99f\n    #\n    # By default Elixir migrations lock the database migration table, so they\n    # will only run from a single instance.\n    migrations =\n      @repos\n      |> Enum.map(&Ecto.Migrator.migrations/1)\n      |> List.flatten()\n\n    if Enum.empty?(migrations) do\n      liveness()\n    else\n      {:error, \"Database not migrated\"}\n    end\n  end\n\n  @doc \"\"\"\n  Check if the app is alive and working properly.\n\n  This returns app status for the Kubernetes `livenessProbe`.\n  Kubernetes continuously checks if the app is alive and working as expected.\n  If it crashes or becomes unresponsive for a specified period of time,\n  Kubernetes kills and replaces the container.\n\n  This check should be lightweight, only determining if the server is\n  responding to requests and can connect to the DB.\n  \"\"\"\n  @spec liveness :: check_return()\n  def liveness do\n    case Ecto.Adapters.SQL.query(Repo, \"SELECT 1\") do\n      {:ok, %{num_rows: 1, rows: [[1]]}} ->\n        :ok\n\n      {:error, reason} ->\n        {:error, inspect(reason)}\n    end\n  rescue\n    e ->\n      {:error, inspect(e)}\n  end\n\n  @doc \"\"\"\n  Check if app should be serving public traffic.\n\n  This returns app status for the Kubernetes `readinessProbe`.\n  Kubernetes continuously checks if the app should serve traffic. If the\n  readiness probe fails, Kubernetes doesn't kill and restart the container,\n  instead it marks the pod as \"unready\" and stops sending traffic to it, e.g.\n  in the ingress.\n\n  This is useful to temporarily stop serving requests. For example, if the app\n  gets a timeout connecting to a back end service, it might return an error for\n  the readiness probe. After multiple failed attempts, it would switch to\n  returning false for the `livenessProbe`, triggering a restart.\n\n  Similarly, the app might return an error if it is overloaded, shedding\n  traffic until it has caught up.\n  \"\"\"\n  @spec readiness :: check_return()\n  def readiness do\n    liveness()\n  end\n\n  @spec basic :: check_return()\n  def basic do\n    :ok\n  end\nend\n```\n\nDocs can be found at  .","ref":"readme.html#usage"},{"type":"extras","title":"Changelog","doc":"# Changelog\nAll notable changes to this project will be documented in this file.\n\nThe format is based on [Keep a Changelog](https://keepachangelog.com/en/1.0.0/),\nand this project adheres to [Semantic Versioning](https://semver.org/spec/v2.0.0.html).\n\n## [Unreleased]\n\n## [0.7.0] - 2024-12-11","ref":"changelog.html"},{"type":"extras","title":"Added - Changelog","doc":"- Initial release","ref":"changelog.html#added"},{"type":"extras","title":"License (Apache-2.0)","doc":"# Apache License\nVersion 2.0, January 2004\n\nhttp://www.apache.org/licenses/\n\nTERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION","ref":"license.html"},{"type":"extras","title":"1. Definitions. - License (Apache-2.0)","doc":"\"License\" shall mean the terms and conditions for use, reproduction, and\ndistribution as defined by Sections 1 through 9 of this document.\n\n\"Licensor\" shall mean the copyright owner or entity authorized by the copyright\nowner that is granting the License.\n\n\"Legal Entity\" shall mean the union of the acting entity and all other entities\nthat control, are controlled by, or are under common control with that entity.\nFor the purposes of this definition, \"control\" means (i) the power, direct or\nindirect, to cause the direction or management of such entity, whether by\ncontract or otherwise, or (ii) ownership of fifty percent (50%) or more of the\noutstanding shares, or (iii) beneficial ownership of such entity.\n\n\"You\" (or \"Your\") shall mean an individual or Legal Entity exercising\npermissions granted by this License.\n\n\"Source\" form shall mean the preferred form for making modifications, including\nbut not limited to software source code, documentation source, and configuration\nfiles.\n\n\"Object\" form shall mean any form resulting from mechanical transformation or\ntranslation of a Source form, including but not limited to compiled object code,\ngenerated documentation, and conversions to other media types.\n\n\"Work\" shall mean the work of authorship, whether in Source or Object form, made\navailable under the License, as indicated by a copyright notice that is included\nin or attached to the work (an example is provided in the Appendix below).\n\n\"Derivative Works\" shall mean any work, whether in Source or Object form, that\nis based on (or derived from) the Work and for which the editorial revisions,\nannotations, elaborations, or other modifications represent, as a whole, an\noriginal work of authorship. For the purposes of this License, Derivative Works\nshall not include works that remain separable from, or merely link (or bind by\nname) to the interfaces of, the Work and Derivative Works thereof.\n\n\"Contribution\" shall mean any work of authorship, including the original version\nof the Work and any modifications or additions to that Work or Derivative Works\nthereof, that is intentionally submitted to Licensor for inclusion in the Work\nby the copyright owner or by an individual or Legal Entity authorized to submit\non behalf of the copyright owner. For the purposes of this definition,\n\"submitted\" means any form of electronic, verbal, or written communication sent\nto the Licensor or its representatives, including but not limited to\ncommunication on electronic mailing lists, source code control systems, and\nissue tracking systems that are managed by, or on behalf of, the Licensor for\nthe purpose of discussing and improving the Work, but excluding communication\nthat is conspicuously marked or otherwise designated in writing by the copyright\nowner as \"Not a Contribution.\"\n\n\"Contributor\" shall mean Licensor and any individual or Legal Entity on behalf\nof whom a Contribution has been received by Licensor and subsequently\nincorporated within the Work.","ref":"license.html#1-definitions"},{"type":"extras","title":"2. Grant of Copyright License. - License (Apache-2.0)","doc":"Subject to the terms and conditions of this License, each Contributor hereby\ngrants to You a perpetual, worldwide, non-exclusive, no-charge, royalty-free,\nirrevocable copyright license to reproduce, prepare Derivative Works of,\npublicly display, publicly perform, sublicense, and distribute the Work and such\nDerivative Works in Source or Object form.","ref":"license.html#2-grant-of-copyright-license"},{"type":"extras","title":"3. Grant of Patent License. - License (Apache-2.0)","doc":"Subject to the terms and conditions of this License, each Contributor hereby\ngrants to You a perpetual, worldwide, non-exclusive, no-charge, royalty-free,\nirrevocable (except as stated in this section) patent license to make, have\nmade, use, offer to sell, sell, import, and otherwise transfer the Work, where\nsuch license applies only to those patent claims licensable by such Contributor\nthat are necessarily infringed by their Contribution(s) alone or by combination\nof their Contribution(s) with the Work to which such Contribution(s) was\nsubmitted. If You institute patent litigation against any entity (including a\ncross-claim or counterclaim in a lawsuit) alleging that the Work or a\nContribution incorporated within the Work constitutes direct or contributory\npatent infringement, then any patent licenses granted to You under this License\nfor that Work shall terminate as of the date such litigation is filed.","ref":"license.html#3-grant-of-patent-license"},{"type":"extras","title":"4. Redistribution. - License (Apache-2.0)","doc":"You may reproduce and distribute copies of the Work or Derivative Works thereof\nin any medium, with or without modifications, and in Source or Object form,\nprovided that You meet the following conditions:\n\n1. You must give any other recipients of the Work or Derivative Works a copy of\n   this License; and\n\n2. You must cause any modified files to carry prominent notices stating that\n   You changed the files; and\n\n3. You must retain, in the Source form of any Derivative Works that You\n   distribute, all copyright, patent, trademark, and attribution notices from\n   the Source form of the Work, excluding those notices that do not pertain to\n   any part of the Derivative Works; and\n\n4. If the Work includes a \"NOTICE\" text file as part of its distribution, then\n   any Derivative Works that You distribute must include a readable copy of the\n   attribution notices contained within such NOTICE file, excluding those\n   notices that do not pertain to any part of the Derivative Works, in at least\n   one of the following places: within a NOTICE text file distributed as part\n   of the Derivative Works; within the Source form or documentation, if\n   provided along with the Derivative Works; or, within a display generated by\n   the Derivative Works, if and wherever such third-party notices normally\n   appear. The contents of the NOTICE file are for informational purposes only\n   and do not modify the License. You may add Your own attribution notices\n   within Derivative Works that You distribute, alongside or as an addendum to\n   the NOTICE text from the Work, provided that such additional attribution\n   notices cannot be construed as modifying the License.\n\nYou may add Your own copyright statement to Your modifications and may provide\nadditional or different license terms and conditions for use, reproduction, or\ndistribution of Your modifications, or for any such Derivative Works as a whole,\nprovided Your use, reproduction, and distribution of the Work otherwise complies\nwith the conditions stated in this License.","ref":"license.html#4-redistribution"},{"type":"extras","title":"5. Submission of Contributions. - License (Apache-2.0)","doc":"Unless You explicitly state otherwise, any Contribution intentionally submitted\nfor inclusion in the Work by You to the Licensor shall be under the terms and\nconditions of this License, without any additional terms or conditions.\nNotwithstanding the above, nothing herein shall supersede or modify the terms of\nany separate license agreement you may have executed with Licensor regarding\nsuch Contributions.","ref":"license.html#5-submission-of-contributions"},{"type":"extras","title":"6. Trademarks. - License (Apache-2.0)","doc":"This License does not grant permission to use the trade names, trademarks,\nservice marks, or product names of the Licensor, except as required for\nreasonable and customary use in describing the origin of the Work and\nreproducing the content of the NOTICE file.","ref":"license.html#6-trademarks"},{"type":"extras","title":"7. Disclaimer of Warranty. - License (Apache-2.0)","doc":"Unless required by applicable law or agreed to in writing, Licensor provides the\nWork (and each Contributor provides its Contributions) on an \"AS IS\" BASIS,\nWITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied,\nincluding, without limitation, any warranties or conditions of TITLE, NON-\nINFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A PARTICULAR PURPOSE. You are\nsolely responsible for determining the appropriateness of using or\nredistributing the Work and assume any risks associated with Your exercise of\npermissions under this License.","ref":"license.html#7-disclaimer-of-warranty"},{"type":"extras","title":"8. Limitation of Liability. - License (Apache-2.0)","doc":"In no event and under no legal theory, whether in tort (including negligence),\ncontract, or otherwise, unless required by applicable law (such as deliberate\nand grossly negligent acts) or agreed to in writing, shall any Contributor be\nliable to You for damages, including any direct, indirect, special, incidental,\nor consequential damages of any character arising as a result of this License or\nout of the use or inability to use the Work (including but not limited to\ndamages for loss of goodwill, work stoppage, computer failure or malfunction, or\nany and all other commercial damages or losses), even if such Contributor has\nbeen advised of the possibility of such damages.","ref":"license.html#8-limitation-of-liability"},{"type":"extras","title":"9. Accepting Warranty or Additional Liability. - License (Apache-2.0)","doc":"While redistributing the Work or Derivative Works thereof, You may choose to\noffer, and charge a fee for, acceptance of support, warranty, indemnity, or\nother liability obligations and/or rights consistent with this License. However,\nin accepting such obligations, You may act only on Your own behalf and on Your\nsole responsibility, not on behalf of any other Contributor, and only if You\nagree to indemnify, defend, and hold each Contributor harmless for any liability\nincurred by, or claims asserted against, such Contributor by reason of your\naccepting any such warranty or additional liability.\n\nEND OF TERMS AND CONDITIONS\n\nCopyright 2021-2023, Jake Morrison  .\n\nLicensed under the Apache License, Version 2.0 (the \"License\");\nyou may not use this file except in compliance with the License.\nYou may obtain a copy of the License at\n\nhttp://www.apache.org/licenses/LICENSE-2.0\n\nUnless required by applicable law or agreed to in writing, software\ndistributed under the License is distributed on an \"AS IS\" BASIS,\nWITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\nSee the License for the specific language governing permissions and\nlimitations under the License.","ref":"license.html#9-accepting-warranty-or-additional-liability"},{"type":"extras","title":"Contributing","doc":"# Contributing to Uinta\n\nPlease take a moment to review this document in order to make the contribution\nprocess easy and effective for everyone involved!\nAlso make sure you read our [Code of Conduct](CODE_OF_CONDUCT.md) that outlines our commitment towards an open and welcoming environment.","ref":"contributing.html"},{"type":"extras","title":"Using the issue tracker - Contributing","doc":"Use the issues tracker for:\n\n* [Bug reports](#bug-reports)\n* [Submitting pull requests](#pull-requests)\n\nWe do our best to keep the issue tracker tidy and organized, making it useful\nfor everyone. For example, we classify open issues per perceived difficulty,\nmaking it easier for developers to [contribute to Uinta](#pull-requests).","ref":"contributing.html#using-the-issue-tracker"},{"type":"extras","title":"Bug reports - Contributing","doc":"A bug is either a _demonstrable problem_ that is caused by the code in the repository,\nor indicate missing, unclear, or misleading documentation. Good bug reports are extremely\nhelpful - thank you!\n\nGuidelines for bug reports:\n\n1. **Use the GitHub issue search** &mdash; check if the issue has already been\n   reported.\n\n2. **Check if the issue has been fixed** &mdash; try to reproduce it using the\n   `master` branch in the repository.\n\n3. **Isolate and report the problem** &mdash; ideally create a reduced test\n   case.\n\nPlease try to be as detailed as possible in your report. Include information about\nyour Operating System, as well as your Erlang, Elixir and Uinta versions. Please provide steps to\nreproduce the issue as well as the outcome you were expecting! All these details\nwill help developers to fix any potential bugs.\n\nExample:\n\n> Short and descriptive example bug report title\n>\n> A summary of the issue and the environment in which it occurs. If suitable,\n> include the steps required to reproduce the bug.\n>\n> 1. This is the first step\n> 2. This is the second step\n> 3. Further steps, etc.\n>\n> ` ` - a link to the reduced test case (e.g. a GitHub Gist)\n>\n> Any other information you want to share that is relevant to the issue being\n> reported. This might include the lines of code that you have identified as\n> causing the bug, and potential solutions (and your opinions on their\n> merits).","ref":"contributing.html#bug-reports"},{"type":"extras","title":"Contributing Documentation - Contributing","doc":"Code documentation (`@doc`, `@moduledoc`, `@typedoc`) has a special convention:\nthe first paragraph is considered to be a short summary.\n\nFor functions, macros and callbacks say what it will do. For example write\nsomething like:\n\n```elixir\n@doc \"\"\"\nMarks the given value as HTML safe.\n\"\"\"\ndef safe({:safe, value}), do: {:safe, value}\n```\n\nFor modules, protocols and types say what it is. For example write\nsomething like:\n\n```elixir\ndefmodule MyModule do\n  @moduledoc \"\"\"\n  Conveniences for working HTML strings and templates.\n  ...\n  \"\"\"\n```\n\nKeep in mind that the first paragraph might show up in a summary somewhere, long\ntexts in the first paragraph create very ugly summaries. As a rule of thumb\nanything longer than 80 characters is too long.\n\nTry to keep unnecessary details out of the first paragraph, it's only there to\ngive a user a quick idea of what the documented \"thing\" does/is. The rest of the\ndocumentation string can contain the details, for example when a value and when\n`nil` is returned.\n\nIf possible include examples, preferably in a form that works with doctests.\nThis makes it easy to test the examples so that they don't go stale and examples\nare often a great help in explaining what a function does.","ref":"contributing.html#contributing-documentation"},{"type":"extras","title":"Pull requests - Contributing","doc":"Good pull requests - patches, improvements, new features - are a fantastic\nhelp. They should remain focused in scope and avoid containing unrelated\ncommits.\n\n**IMPORTANT**: By submitting a patch, you agree that your work will be\nlicensed under the license used by the project.\n\nIf you have any large pull request in mind (e.g. implementing features,\nrefactoring code, etc), **please ask first** otherwise you risk spending\na lot of time working on something that the project's developers might\nnot want to merge into the project.\n\nPlease adhere to the coding conventions in the project (indentation,\naccurate comments, etc.) and don't forget to add your own tests and\ndocumentation. When working with git, we recommend the following process\nin order to craft an excellent pull request:\n\n1. [Fork](https://help.github.com/articles/fork-a-repo/) the project, clone your fork,\n   and configure the remotes:\n\n   ```bash\n   # Clone your fork of the repo into the current directory\n   git clone https://github.com/ /uinta\n\n   # Navigate to the newly cloned directory\n   cd uinta\n\n   # Assign the original repo to a remote called \"upstream\"\n   git remote add upstream https://github.com/podium/uinta\n   ```\n\n2. If you cloned a while ago, get the latest changes from upstream, and update your fork:\n\n   ```bash\n   git checkout master\n   git pull upstream master\n   git push\n   ```\n\n3. Create a new topic branch (off of `master`) to contain your feature, change,\n   or fix.\n\n   **IMPORTANT**: Making changes in `master` is discouraged. You should always\n   keep your local `master` in sync with upstream `master` and make your\n   changes in topic branches.\n\n   ```bash\n   git checkout -b  \n   ```\n\n4. Commit your changes in logical chunks. Keep your commit messages organized,\n   with a short description in the first line and more detailed information on\n   the following lines. Feel free to use Git's\n   [interactive rebase](https://help.github.com/articles/about-git-rebase/)\n   feature to tidy up your commits before making them public.\n\n5. Make sure all the tests are still passing.\n\n   ```bash\n   mix test\n   ```\n\n6. Push your topic branch up to your fork:\n\n   ```bash\n   git push origin  \n   ```\n\n7. [Open a Pull Request](https://help.github.com/articles/about-pull-requests/)\n    with a clear title and description.\n\n8. If you haven't updated your pull request for a while, you should consider\n   rebasing on master and resolving any conflicts.\n\n   **IMPORTANT**: _Never ever_ merge upstream `master` into your branches. You\n   should always `git rebase` on `master` to bring your changes up to date when\n   necessary.\n\n   ```bash\n   git checkout master\n   git pull upstream master\n   git checkout  \n   git rebase master\n   ```\n\nThank you for your contributions!","ref":"contributing.html#pull-requests"},{"type":"extras","title":"Guides - Contributing","doc":"These Guides aim to be inclusive. We use \"we\" and \"our\" instead of \"you\" and\n\"your\" to foster this sense of inclusion.\n\nIdeally there is something for everybody in each guide, from beginner to expert.\nThis is hard, maybe impossible. When we need to compromise, we do so on behalf\nof beginning users because expert users have more tools at their disposal to\nhelp themselves.\n\nThe general pattern we use for presenting information is to first introduce a\nsmall, discrete topic, then write a small amount of code to demonstrate the\nconcept, then verify that the code worked.\n\nIn this way, we build from small, easily digestible concepts into more complex\nones. The shorter this cycle is, as long as the information is still clear and\ncomplete, the better.\n\nFor formatting the guides:\n\n- We use the `elixir` code fence for all module code.\n- We use the `iex` for IEx sessions.\n- We use the `console` code fence for shell commands.\n- We use the `html` code fence for html templates, even if there is elixir code\n  in the template.\n- We use backticks for filenames and directory paths.\n- We use backticks for module names, function names, and variable names.\n- Documentation line length should hard wrapped at around 100 characters if possible.","ref":"contributing.html#guides"},{"type":"extras","title":"Code of Conduct","doc":"# Contributor Covenant Code of Conduct","ref":"code_of_conduct.html"},{"type":"extras","title":"Our Pledge - Code of Conduct","doc":"We as members, contributors, and leaders pledge to make participation in our\ncommunity a harassment-free experience for everyone, regardless of age, body\nsize, visible or invisible disability, ethnicity, sex characteristics, gender\nidentity and expression, level of experience, education, socio-economic status,\nnationality, personal appearance, race, caste, color, religion, or sexual\nidentity and orientation.\n\nWe pledge to act and interact in ways that contribute to an open, welcoming,\ndiverse, inclusive, and healthy community.","ref":"code_of_conduct.html#our-pledge"},{"type":"extras","title":"Our Standards - Code of Conduct","doc":"Examples of behavior that contributes to a positive environment for our\ncommunity include:\n\n* Demonstrating empathy and kindness toward other people\n* Being respectful of differing opinions, viewpoints, and experiences\n* Giving and gracefully accepting constructive feedback\n* Accepting responsibility and apologizing to those affected by our mistakes,\n  and learning from the experience\n* Focusing on what is best not just for us as individuals, but for the overall\n  community\n\nExamples of unacceptable behavior include:\n\n* The use of sexualized language or imagery, and sexual attention or advances of\n  any kind\n* Trolling, insulting or derogatory comments, and personal or political attacks\n* Public or private harassment\n* Publishing others' private information, such as a physical or email address,\n  without their explicit permission\n* Other conduct which could reasonably be considered inappropriate in a\n  professional setting","ref":"code_of_conduct.html#our-standards"},{"type":"extras","title":"Enforcement Responsibilities - Code of Conduct","doc":"Community leaders are responsible for clarifying and enforcing our standards of\nacceptable behavior and will take appropriate and fair corrective action in\nresponse to any behavior that they deem inappropriate, threatening, offensive,\nor harmful.\n\nCommunity leaders have the right and responsibility to remove, edit, or reject\ncomments, commits, code, wiki edits, issues, and other contributions that are\nnot aligned to this Code of Conduct, and will communicate reasons for moderation\ndecisions when appropriate.","ref":"code_of_conduct.html#enforcement-responsibilities"},{"type":"extras","title":"Scope - Code of Conduct","doc":"This Code of Conduct applies within all community spaces, and also applies when\nan individual is officially representing the community in public spaces.\nExamples of representing our community include using an official e-mail address,\nposting via an official social media account, or acting as an appointed\nrepresentative at an online or offline event.","ref":"code_of_conduct.html#scope"},{"type":"extras","title":"Enforcement - Code of Conduct","doc":"Instances of abusive, harassing, or otherwise unacceptable behavior may be\nreported to the community leaders responsible for enforcement at\nengineering@therealreal.com. All complaints will be reviewed and investigated\npromptly and fairly.\n\nAll community leaders are obligated to respect the privacy and security of the\nreporter of any incident.","ref":"code_of_conduct.html#enforcement"},{"type":"extras","title":"Enforcement Guidelines - Code of Conduct","doc":"Community leaders will follow these Community Impact Guidelines in determining\nthe consequences for any action they deem in violation of this Code of Conduct:","ref":"code_of_conduct.html#enforcement-guidelines"},{"type":"extras","title":"1. Correction - Code of Conduct","doc":"**Community Impact**: Use of inappropriate language or other behavior deemed\nunprofessional or unwelcome in the community.\n\n**Consequence**: A private, written warning from community leaders, providing\nclarity around the nature of the violation and an explanation of why the\nbehavior was inappropriate. A public apology may be requested.","ref":"code_of_conduct.html#1-correction"},{"type":"extras","title":"2. Warning - Code of Conduct","doc":"**Community Impact**: A violation through a single incident or series of\nactions.\n\n**Consequence**: A warning with consequences for continued behavior. No\ninteraction with the people involved, including unsolicited interaction with\nthose enforcing the Code of Conduct, for a specified period of time. This\nincludes avoiding interactions in community spaces as well as external channels\nlike social media. Violating these terms may lead to a temporary or permanent\nban.","ref":"code_of_conduct.html#2-warning"},{"type":"extras","title":"3. Temporary Ban - Code of Conduct","doc":"**Community Impact**: A serious violation of community standards, including\nsustained inappropriate behavior.\n\n**Consequence**: A temporary ban from any sort of interaction or public\ncommunication with the community for a specified period of time. No public or\nprivate interaction with the people involved, including unsolicited interaction\nwith those enforcing the Code of Conduct, is allowed during this period.\nViolating these terms may lead to a permanent ban.","ref":"code_of_conduct.html#3-temporary-ban"},{"type":"extras","title":"4. Permanent Ban - Code of Conduct","doc":"**Community Impact**: Demonstrating a pattern of violation of community\nstandards, including sustained inappropriate behavior, harassment of an\nindividual, or aggression toward or disparagement of classes of individuals.\n\n**Consequence**: A permanent ban from any sort of public interaction within the\ncommunity.","ref":"code_of_conduct.html#4-permanent-ban"},{"type":"extras","title":"Attribution - Code of Conduct","doc":"This Code of Conduct is adapted from the [Contributor Covenant][homepage],\nversion 2.1, available at\n[https://www.contributor-covenant.org/version/2/1/code_of_conduct.html][v2.1].\n\nCommunity Impact Guidelines were inspired by\n[Mozilla's code of conduct enforcement ladder][Mozilla CoC].\n\nFor answers to common questions about this code of conduct, see the FAQ at\n[https://www.contributor-covenant.org/faq][FAQ]. Translations are available at\n[https://www.contributor-covenant.org/translations][translations].\n\n[homepage]: https://www.contributor-covenant.org\n[v2.1]: https://www.contributor-covenant.org/version/2/1/code_of_conduct.html\n[Mozilla CoC]: https://github.com/mozilla/diversity\n[FAQ]: https://www.contributor-covenant.org/faq\n[translations]: https://www.contributor-covenant.org/translations","ref":"code_of_conduct.html#attribution"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.35.1"}}